@@ -62,11 +62,11 @@ var (
 	halfCloseFlag   = flag.Bool("halfclose", false,
 		"When not keepalive, whether to half close the connection (only for fast http)")
 	httpReqTimeoutFlag  = flag.Duration("timeout", fhttp.HTTPReqTimeOutDefaultValue, "Connection and read timeout value (for http)")
-	stdClientFlag       = flag.Bool("stdclient", false, "Use the slower net/http standard client (works for TLS)")
+	stdClientFlag       = flag.Bool("stdclient", false, "Use the slower net/http standard client (slower but supports h2)")
 	http10Flag          = flag.Bool("http1.0", false, "Use http1.0 (instead of http 1.1)")
-	httpsInsecureFlag   = flag.Bool("k", false, "Do not verify certs in https connections")
+	httpsInsecureFlag   = flag.Bool("k", false, "Do not verify certs in https/tls/grpc connections")
 	httpsInsecureFlagL  = flag.Bool("https-insecure", false, "Long form of the -k flag")
-	resolve             = flag.String("resolve", "", "Resolve CN of cert to this IP, so that we can call https://cn directly")
+	resolve             = flag.String("resolve", "", "Resolve host name to this `IP`")
 	headersFlags        headersFlagList
 	httpOpts            fhttp.HTTPOptions
 	followRedirectsFlag = flag.Bool("L", false, "Follow redirects (implies -std-client) - do not use for load test")
@@ -100,6 +100,12 @@ var (
 	LogErrorsFlag = flag.Bool("log-errors", true, "Log http non 2xx/418 error codes as they occur")
 	// RunIDFlag is optional RunID to be present in json results (and default json result filename if not 0).
 	RunIDFlag = flag.Int64("runid", 0, "Optional RunID to add to json result and auto save filename, to match server mode")
+	// HelpFlag is true if help/usage is being requested by the user.
+	HelpFlag   = flag.Bool("h", false, "Print usage/help on stdout")
+	warmupFlag = flag.Bool("sequential-warmup", false,
+		"http(s) runner warmup done in parallel instead of sequentially. When set, restores pre 1.21 behavior")
+	curlHeadersStdout = flag.Bool("curl-stdout-headers", false,
+		"Restore pre 1.22 behavior where http headers of the fast client are output to stdout in curl mode. now stderr by default.")
 )
 
 // SharedMain is the common part of main from fortio_main and fcurl.
@@ -113,7 +119,8 @@ func SharedMain(usage func(io.Writer, ...interface{})) {
 	if len(os.Args) < 2 {
 		return
 	}
-	if strings.Contains(os.Args[1], "version") {
+	firstArg := os.Args[1]
+	if strings.Contains(firstArg, "version") {
 		if len(os.Args) >= 3 && strings.Contains(os.Args[2], "s") {
 			// so `fortio version -s` is the short version; everything else is long/full
 			fmt.Println(version.Short())
@@ -122,7 +129,7 @@ func SharedMain(usage func(io.Writer, ...interface{})) {
 		}
 		os.Exit(0)
 	}
-	if strings.Contains(os.Args[1], "help") {
+	if strings.Contains(firstArg, "help") || firstArg == "-h" {
 		usage(os.Stdout)
 		os.Exit(0)
 	}
@@ -140,7 +147,12 @@ func FetchURL(o *fhttp.HTTPOptions) {
 	}
 	code, data, header := client.Fetch()
 	log.LogVf("Fetch result code %d, data len %d, headerlen %d", code, len(data), header)
-	os.Stdout.Write(data)
+	if *curlHeadersStdout {
+		os.Stdout.Write(data)
+	} else {
+		os.Stderr.Write(data[:header])
+		os.Stdout.Write(data[header:])
+	}
 	if code != http.StatusOK {
 		log.Errf("Error status %d : %s", code, fhttp.DebugSummary(data, 512))
 		os.Exit(1)
@@ -183,5 +195,6 @@ func SharedHTTPOptions() *fhttp.HTTPOptions {
 	httpOpts.Cert = *CertFlag
 	httpOpts.Key = *KeyFlag
 	httpOpts.LogErrors = *LogErrorsFlag
+	httpOpts.SequentialWarmup = *warmupFlag
 	return &httpOpts
 }
