@@ -19,7 +19,6 @@ import (
 	"bytes"
 	"context"
 	"crypto/tls"
-	"crypto/x509"
 	"errors"
 	"fmt"
 	"io"
@@ -142,10 +141,6 @@ func (h *HTTPOptions) URLSchemeCheck() {
 	}
 	if strings.HasPrefix(lcURL, hs) {
 		h.https = true
-		if !h.DisableFastClient {
-			log.Warnf("https requested, switching to standard go client")
-			h.DisableFastClient = true
-		}
 		return // url is good
 	}
 	if !strings.HasPrefix(lcURL, "http://") {
@@ -164,6 +159,7 @@ const (
 
 // HTTPOptions holds the common options of both http clients and the headers.
 type HTTPOptions struct {
+	TLSOptions
 	URL               string
 	NumConnections    int  // num connections (for std client)
 	Compression       bool // defaults to no compression, only used by std client
@@ -171,27 +167,21 @@ type HTTPOptions struct {
 	HTTP10            bool // defaults to http1.1
 	DisableKeepAlive  bool // so default is keep alive
 	AllowHalfClose    bool // if not keepalive, whether to half close after request
-	Insecure          bool // do not verify certs for https
 	FollowRedirects   bool // For the Std Client only: follow redirects.
 	initDone          bool
 	https             bool   // whether URLSchemeCheck determined this was an https:// call or not
-	CACert            string // `Path` to a custom CA certificate file to be used
-	Cert              string // `Path` to the certificate file to be used
-	Key               string // `Path` to the key file used
 	Resolve           string // resolve Common Name to this ip when use CN as target url
 	// ExtraHeaders to be added to each request (UserAgent and headers set through AddAndValidateExtraHeader()).
 	extraHeaders http.Header
 	// Host is treated specially, remember that virtual header separately.
-	hostOverride   string
-	HTTPReqTimeOut time.Duration // timeout value for http request
-
-	UserCredentials string // user credentials for authorization
-	ContentType     string // indicates request body type, implies POST instead of GET
-	Payload         []byte // body for http request, implies POST if not empty.
-
-	UnixDomainSocket string // Path of unix domain socket to use instead of host:port from URL
-	LogErrors        bool   // whether to log non 2xx code as they occur or not
-	ID               int    // id to use for logging (thread id when used as a runner)
+	hostOverride     string
+	HTTPReqTimeOut   time.Duration // timeout value for http request
+	UserCredentials  string        // user credentials for authorization
+	ContentType      string        // indicates request body type, implies POST instead of GET
+	Payload          []byte        // body for http request, implies POST if not empty.
+	LogErrors        bool          // whether to log non 2xx code as they occur or not
+	ID               int           // id to use for logging (thread id when used as a runner)
+	SequentialWarmup bool          // whether to do http(s) runs warmup sequentially or in parallel (new default is //)
 }
 
 // ResetHeaders resets all the headers, including the User-Agent: one (and the Host: logical special header).
@@ -370,11 +360,14 @@ func (c *Client) Fetch() (int, []byte, int) {
 		bodyBytes := []byte(body)
 		c.req.ContentLength = int64(len(bodyBytes))
 		c.req.Body = ioutil.NopCloser(bytes.NewReader(bodyBytes))
+	} else if c.body != "" {
+		c.req.Body = ioutil.NopCloser(bytes.NewReader([]byte(c.body)))
 	}
+
 	resp, err := c.client.Do(c.req)
 	if err != nil {
 		log.Errf("[%d] Unable to send %s request for %s : %v", c.id, c.req.Method, c.url, err)
-		return http.StatusBadRequest, []byte(err.Error()), 0
+		return -1, []byte(err.Error()), 0
 	}
 	var data []byte
 	if log.LogDebug() {
@@ -439,34 +432,12 @@ func NewStdClient(o *HTTPOptions) (*Client, error) {
 		},
 		TLSHandshakeTimeout: o.HTTPReqTimeOut,
 	}
-	if o.https { // nolint: nestif // fine for now
-		tr.TLSClientConfig = &tls.Config{MinVersion: tls.VersionTLS12}
-		if o.Insecure {
-			log.LogVf("Using insecure https")
-			tr.TLSClientConfig.InsecureSkipVerify = true
-		}
-		if len(o.Cert) > 0 && len(o.Key) > 0 {
-			cert, err := tls.LoadX509KeyPair(o.Cert, o.Key)
-			if err != nil {
-				log.Errf("LoadX509KeyPair error for cert %v / key %v: %v", o.Cert, o.Key, err)
-				return nil, err
-			}
-			tr.TLSClientConfig.Certificates = []tls.Certificate{cert}
-		}
-		if len(o.CACert) > 0 {
-			// Load CA cert
-			caCert, err := ioutil.ReadFile(o.CACert)
-			if err != nil {
-				log.Errf("Unable to read CA from %v: %v", o.CACert, err)
-				return nil, err
-			}
-			log.LogVf("Using custom CA from %v", o.CACert)
-			caCertPool := x509.NewCertPool()
-			caCertPool.AppendCertsFromPEM(caCert)
-			tr.TLSClientConfig.RootCAs = caCertPool
+	if o.https {
+		tr.TLSClientConfig, err = o.TLSOptions.TLSClientConfig()
+		if err != nil {
+			return nil, err
 		}
 	}
-
 	client := Client{
 		url:                  o.URL,
 		path:                 req.URL.Path,
@@ -536,6 +507,8 @@ type FastClient struct {
 	uuidMarkers  [][]byte
 	logErrors    bool
 	id           int
+	https        bool
+	tlsConfig    *tls.Config
 }
 
 // Close cleans up any resources used by FastClient.
@@ -583,14 +556,17 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) {
 		log.Errf("Bad url '%s' : %v", urlString, err)
 		return nil, err
 	}
-	if url.Scheme != "http" {
-		log.Errf("Only http is supported with the optimized client, use -stdclient for url %s", o.URL)
-		return nil, fmt.Errorf("only http for fast client")
-	}
 	// note: Host includes the port
 	bc := FastClient{
 		url: o.URL, host: url.Host, hostname: url.Hostname(), port: url.Port(),
 		http10: o.HTTP10, halfClose: o.AllowHalfClose, logErrors: o.LogErrors, id: o.ID,
+		https: o.https,
+	}
+	if o.https {
+		bc.tlsConfig, err = o.TLSOptions.TLSClientConfig()
+		if err != nil {
+			return nil, err
+		}
 	}
 	bc.buffer = make([]byte, BufferSizeKb*1024)
 	if bc.port == "" {
@@ -623,6 +599,9 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) {
 	if customHostHeader {
 		host = o.hostOverride
 	}
+	if bc.tlsConfig != nil {
+		bc.tlsConfig.ServerName = host
+	}
 	var buf bytes.Buffer
 	buf.WriteString(method + " " + url.RequestURI() + " HTTP/" + proto + "\r\n")
 	if !bc.http10 || customHostHeader {
@@ -666,10 +645,20 @@ func (c *FastClient) returnRes() (int, []byte, int) {
 // connect to destination.
 func (c *FastClient) connect() net.Conn {
 	c.socketCount++
-	socket, err := net.Dial(c.dest.Network(), c.dest.String())
-	if err != nil {
-		log.Errf("Unable to connect to %v : %v", c.dest, err)
-		return nil
+	var socket net.Conn
+	var err error
+	if c.https {
+		socket, err = tls.Dial(c.dest.Network(), c.dest.String(), c.tlsConfig)
+		if err != nil {
+			log.Errf("[%d] Unable to TLS connect to %v : %v", c.id, c.dest, err)
+			return nil
+		}
+	} else {
+		socket, err = net.Dial(c.dest.Network(), c.dest.String())
+		if err != nil {
+			log.Errf("[%d] Unable to connect to %v : %v", c.id, c.dest, err)
+			return nil
+		}
 	}
 	fnet.SetSocketBuffers(socket, len(c.buffer), len(c.req))
 	return socket
@@ -712,28 +701,28 @@ func (c *FastClient) Fetch() (int, []byte, int) {
 	if err != nil || conErr != nil {
 		if reuse {
 			// it's ok for the (idle) socket to die once, auto reconnect:
-			log.Infof("Closing dead socket %v (%v)", conn, err)
+			log.Infof("[%d] Closing dead socket %v (%v)", c.id, c.dest, err)
 			conn.Close()
 			c.errorCount++
 			return c.Fetch() // recurse once
 		}
-		log.Errf("Unable to write to %v %v : %v", conn, c.dest, err)
+		log.Errf("[%d] Unable to write to %v : %v", c.id, c.dest, err)
 		return c.returnRes()
 	}
 	if n != len(c.req) {
-		log.Errf("Short write to %v %v : %d instead of %d", conn, c.dest, n, len(c.req))
+		log.Errf("[%d] Short write to %v : %d instead of %d", c.id, c.dest, n, len(c.req))
 		return c.returnRes()
 	}
 	if !c.keepAlive && c.halfClose { // nolint: nestif
 		tcpConn, ok := conn.(*net.TCPConn)
 		if ok {
 			if err = tcpConn.CloseWrite(); err != nil {
-				log.Errf("Unable to close write to %v %v : %v", conn, c.dest, err)
+				log.Errf("[%d] Unable to close write to %v : %v", c.id, c.dest, err)
 				return c.returnRes()
 			} // else:
 			log.Debugf("Half closed ok after sending request %v %v", conn, c.dest)
 		} else {
-			log.Warnf("[%d] Unable to close write non tcp connection %v", c.id, conn)
+			log.Warnf("[%d] Unable to close write non tcp connection %v", c.id, c.dest)
 		}
 	}
 	// Read the response:
@@ -752,7 +741,7 @@ func codeIsOK(code int) bool {
 }
 
 // Response reading:
-// nolint: nestif,funlen,gocognit,gocyclo // TODO: refactor - unwiedly/ugly atm.
+// nolint: nestif,funlen,gocognit,gocyclo,maintidx // TODO: refactor - unwiedly/ugly atm.
 func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 	max := len(c.buffer)
 	parsedHeaders := false
@@ -771,11 +760,11 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 			if err != nil {
 				if reusedSocket && c.size == 0 {
 					// Ok for reused socket to be dead once (close by server)
-					log.Infof("Closing dead socket %v (err %v at first read)", conn, err)
+					log.Infof("[%d] Closing dead socket %v (err %v at first read)", c.id, c.dest, err)
 					c.errorCount++
 					err = conn.Close() // close the previous one
 					if err != nil {
-						log.Warnf("[%d] Error closing dead socket %v: %v", c.id, conn, err)
+						log.Warnf("[%d] Error closing dead socket %v: %v", c.id, c.dest, err)
 					}
 					c.code = RetryOnce // special "retry once" code
 					return
@@ -784,7 +773,7 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 					// handled below as possibly normal end of stream after we read something
 					break
 				}
-				log.Errf("Read error %v %v %d : %v", conn, c.dest, c.size, err)
+				log.Errf("[%d] Read error for %v %d : %v", c.id, c.dest, c.size, err)
 				c.code = SocketError
 				break
 			}
@@ -881,7 +870,7 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 					}
 					if checkConnectionClosedHeader {
 						if found, _ := FoldFind(c.buffer[:c.headerLen], connectionCloseHeader); found {
-							log.Infof("Server wants to close connection, no keep-alive!")
+							log.Infof("[%d] Server wants to close connection, no keep-alive!", c.id)
 							keepAlive = false
 							max = len(c.buffer) // reset to read as much as available
 						}
@@ -934,7 +923,7 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 		c.socket = conn // keep the open socket
 	} else {
 		if err := conn.Close(); err != nil {
-			log.Errf("Close error %v %v %d : %v", conn, c.dest, c.size, err)
+			log.Errf("[%d] Close error %v %d : %v", c.id, c.dest, c.size, err)
 		} else {
 			log.Debugf("Closed ok %v from %v after reading %d bytes", conn, c.dest, c.size)
 		}
