@@ -51,6 +51,7 @@ var (
 		"Default parameters/querystring to use if there isn't one provided explicitly. E.g \"status=404&delay=3s\"")
 	fetch2CopiesAllHeader = dflag.DynBool(flag.CommandLine, "proxy-all-headers", true,
 		"Determines if only tracing or all headers (and cookies) are copied from request on the fetch2 ui/server endpoint")
+	serverIdleTimeout = dflag.DynDuration(flag.CommandLine, "server-idle-timeout", 30*time.Second, "Default IdleTimeout for servers")
 )
 
 // EchoHandler is an http server handler echoing back the input.
@@ -97,10 +98,16 @@ func EchoHandler(w http.ResponseWriter, r *http.Request) {
 		rqNum := atomic.AddInt64(&EchoRequests, 1)
 		log.Debugf("Request # %v", rqNum)
 	}
-	if r.FormValue("close") != "" {
+	if generateClose(r.FormValue("close")) {
 		log.Debugf("Adding Connection:close / will close socket")
 		w.Header().Set("Connection", "close")
 	}
+	gzip := strings.Contains(r.Header.Get("Accept-Encoding"), "gzip") && generateGzip(r.FormValue("gzip"))
+	if gzip {
+		gwz := NewGzipHTTPResponseWriter(w)
+		defer gwz.Close()
+		w = gwz
+	}
 	// process header(s) args, must be before size to compose properly
 	for _, hdr := range r.Form["header"] {
 		log.LogVf("Adding requested header %s", hdr)
@@ -167,7 +174,8 @@ func HTTPServer(name string, port string) (*http.ServeMux, net.Addr) {
 	m := http.NewServeMux()
 	h2s := &http2.Server{}
 	s := &http.Server{
-		Handler: h2c.NewHandler(m, h2s),
+		IdleTimeout: serverIdleTimeout.Get(),
+		Handler:     h2c.NewHandler(m, h2s),
 	}
 	listener, addr := fnet.Listen(name, port)
 	if listener == nil {
@@ -337,18 +345,24 @@ func CacheOn(w http.ResponseWriter) {
 	w.Header().Set("Cache-Control", "max-age=365000000, immutable")
 }
 
+// EchoDebugPath returns the additional echo handler path behind debugPath (ie /debug -> /debug/echo/).
+func EchoDebugPath(debugPath string) string {
+	return strings.TrimSuffix(debugPath, "/") + "/echo/"
+}
+
 // Serve starts a debug / echo http server on the given port.
 // Returns the mux and addr where the listening socket is bound.
 // The .Port can be retrieved from it when requesting the 0 port as
 // input for dynamic http server.
 func Serve(port, debugPath string) (*http.ServeMux, net.Addr) {
 	startTime = time.Now()
-	mux, addr := HTTPServer("echo", port)
+	mux, addr := HTTPServer("http-echo", port)
 	if addr == nil {
 		return nil, nil // error already logged
 	}
 	if debugPath != "" {
-		mux.HandleFunc(debugPath, DebugHandler)
+		mux.Handle(debugPath, Gzip(http.HandlerFunc(DebugHandler)))
+		mux.HandleFunc(EchoDebugPath(debugPath), EchoHandler) // Fix #524
 	}
 	mux.HandleFunc("/", EchoHandler)
 	return mux, addr
