@@ -36,7 +36,6 @@ import (
 var uuids map[string]bool
 
 func init() {
-	log.SetLogLevel(log.Debug)
 	uuids = map[string]bool{}
 }
 
@@ -114,27 +113,23 @@ func TestSchemeCheck(t *testing.T) {
 	tests := []struct {
 		input  string
 		output string
-		stdcli bool
 	}{
-		{"https://www.google.com/", "https://www.google.com/", true},
-		{"www.google.com", "http://www.google.com", false},
-		{"hTTps://foo.bar:123/ab/cd", "hTTps://foo.bar:123/ab/cd", true}, // not double http:
-		{"HTTP://foo.bar:124/ab/cd", "HTTP://foo.bar:124/ab/cd", false},  // not double http:
-		{"", "", false},                      // and error in the logs
-		{"x", "http://x", false},             // should not crash because url is shorter than prefix
-		{"http:/", "http://http:/", false},   // boundary
-		{"http://", "http://", false},        // boundary
-		{"https://", "https://", true},       // boundary
-		{"https:/", "http://https:/", false}, // boundary
+		{"https://www.google.com/", "https://www.google.com/"},
+		{"www.google.com", "http://www.google.com"},
+		{"hTTps://foo.bar:123/ab/cd", "hTTps://foo.bar:123/ab/cd"}, // not double http:
+		{"HTTP://foo.bar:124/ab/cd", "HTTP://foo.bar:124/ab/cd"},   // not double http:
+		{"", ""},                      // and error in the logs
+		{"x", "http://x"},             // should not crash because url is shorter than prefix
+		{"http:/", "http://http:/"},   // boundary
+		{"http://", "http://"},        // boundary
+		{"https://", "https://"},      // boundary
+		{"https:/", "http://https:/"}, // boundary
 	}
 	for _, tst := range tests {
 		o := NewHTTPOptions(tst.input)
 		if o.URL != tst.output {
 			t.Errorf("Got %v, expecting %v for url '%s'", o.URL, tst.output, tst.input)
 		}
-		if o.DisableFastClient != tst.stdcli {
-			t.Errorf("Got %v, expecting %v for stdclient for url '%s'", o.DisableFastClient, tst.stdcli, tst.input)
-		}
 	}
 }
 
@@ -497,6 +492,50 @@ func TestGenerateSize(t *testing.T) {
 	}
 }
 
+func TestGenerateClose(t *testing.T) {
+	tests := []struct {
+		input    string
+		expected bool
+	}{
+		// not numbers
+		{"true", true},
+		{"false", false},
+		{"x", true},
+		// Numbers
+		{"0", false},
+		{"0.0", false},
+		{"99.9999999", true}, // well, in theory this should fail once in a blue moon
+		{"100", true},
+	}
+	for _, tst := range tests {
+		if actual := generateClose(tst.input); actual != tst.expected {
+			t.Errorf("Got %v, expected %v for generateClose(%q)", actual, tst.expected, tst.input)
+		}
+	}
+}
+
+func TestGenerateGzip(t *testing.T) {
+	tests := []struct {
+		input    string
+		expected bool
+	}{
+		// not numbers
+		{"true", true},
+		{"false", false},
+		{"x", true},
+		// Numbers
+		{"0", false},
+		{"0.0", false},
+		{"99.9999999", true}, // well, in theory this should fail once in a blue moon
+		{"100", true},
+	}
+	for _, tst := range tests {
+		if actual := generateGzip(tst.input); actual != tst.expected {
+			t.Errorf("Got %v, expected %v for generateGzip(%q)", actual, tst.expected, tst.input)
+		}
+	}
+}
+
 func TestPayloadWithEchoBack(t *testing.T) {
 	tests := []struct {
 		payload           []byte
@@ -528,6 +567,31 @@ func TestPayloadWithEchoBack(t *testing.T) {
 	}
 }
 
+// Test Post request with std client and the socket close after answering.
+func TestPayloadWithStdClientAndClosedSocket(t *testing.T) {
+	m, a := DynamicHTTPServer(false)
+	m.HandleFunc("/", EchoHandler)
+
+	url := fmt.Sprintf("http://localhost:%d/?close=true", a.Port)
+	payload := []byte("{\"test\" : \"test\"}")
+	opts := NewHTTPOptions(url)
+	opts.DisableFastClient = true
+	opts.Payload = payload
+	cli, _ := NewClient(opts)
+
+	for i := 0; i < 3; i++ {
+		code, body, header := cli.Fetch()
+
+		if code != 200 {
+			t.Errorf("Unexpected http status code, expected 200, got %d", code)
+		}
+
+		if !bytes.Equal(body[header:], payload) {
+			t.Errorf("Got %s, expected %q from echo", DebugSummary(body, 512), payload)
+		}
+	}
+}
+
 // Many of the earlier http tests are through httprunner but new tests should go here
 
 func TestUnixDomainHttp(t *testing.T) {
@@ -536,7 +600,7 @@ func TestUnixDomainHttp(t *testing.T) {
 	if addr == nil {
 		t.Fatalf("Error for Serve for %s", uds)
 	}
-	o := HTTPOptions{UnixDomainSocket: uds, URL: "http://foo.bar:123/debug1"}
+	o := HTTPOptions{TLSOptions: TLSOptions{UnixDomainSocket: uds}, URL: "http://foo.bar:123/debug1"}
 	client, _ := NewClient(&o)
 	code, data, _ := client.Fetch()
 	if code != http.StatusOK {
@@ -649,12 +713,7 @@ func TestDefaultPort(t *testing.T) {
 	cli.Close()
 	opts.URL = "https://fortio.org" // will be https port 443
 	opts.Insecure = true            // not needed as we have valid certs but to exercise that code
-	cli, err := NewFastClient(opts)
-	if cli != nil || err == nil {
-		// If https support was added, remove this whitebox/for coverage purpose assertion
-		t.Errorf("fast client isn't supposed to support https (yet), got %v", cli)
-	}
-	cli, err = NewClient(opts)
+	cli, err := NewClient(opts)
 	if cli == nil {
 		t.Fatalf("Couldn't get a client using NewClient on modified opts: %v", err)
 	}
@@ -709,8 +768,8 @@ func TestInvalidRequest(t *testing.T) {
 	client.ChangeURL(" http://bad.url.with.space.com/") // invalid url
 	// should not crash (issue #93), should error out
 	code, _, _ := client.Fetch()
-	if code != http.StatusBadRequest {
-		t.Errorf("Got %d code while expecting bad request (%d)", code, http.StatusBadRequest)
+	if code != -1 {
+		t.Errorf("Got %d code while expecting -1 (local error)", code)
 	}
 	o.URL = client.url
 	c2, err := NewStdClient(&o)
@@ -1369,15 +1428,15 @@ func TestInsecureRequest(t *testing.T) {
 		code       int  // expected code
 	}{
 		{false, true, http.StatusOK},
-		{false, false, http.StatusBadRequest},
+		{false, false, -1},
 		{true, true, http.StatusOK},
-		{true, false, http.StatusBadRequest},
+		{true, false, -1},
 	}
 	for _, tst := range tests {
 		o := HTTPOptions{
 			DisableFastClient: tst.fastClient,
 			URL:               expiredURL,
-			Insecure:          tst.insecure,
+			TLSOptions:        TLSOptions{Insecure: tst.insecure},
 		}
 		code, _ := Fetch(&o)
 		if code != tst.code {
@@ -1400,15 +1459,15 @@ func TestInsecureRequestWithResolve(t *testing.T) {
 		code       int  // expected code
 	}{
 		{false, true, http.StatusOK},
-		{false, false, http.StatusBadRequest},
+		{false, false, -1},
 		{true, true, http.StatusOK},
-		{true, false, http.StatusBadRequest},
+		{true, false, -1},
 	}
 	for _, tst := range tests {
 		o := HTTPOptions{
 			DisableFastClient: tst.fastClient,
 			URL:               url,
-			Insecure:          tst.insecure,
+			TLSOptions:        TLSOptions{Insecure: tst.insecure},
 			Resolve:           "127.0.0.1",
 		}
 		code, _ := Fetch(&o)
