@@ -16,10 +16,12 @@ package fhttp
 
 import (
 	"fmt"
+	"net/http"
 	"os"
 	"runtime"
 	"runtime/pprof"
 	"sort"
+	"sync"
 
 	"fortio.org/fortio/log"
 	"fortio.org/fortio/periodic"
@@ -39,9 +41,9 @@ type HTTPRunnerResults struct {
 	sizes       *stats.Histogram
 	headerSizes *stats.Histogram
 	// exported result
+	HTTPOptions
 	Sizes       *stats.HistogramData
 	HeaderSizes *stats.HistogramData
-	URL         string
 	SocketCount int
 	// http code to abort the run on (-1 for connection or other socket error)
 	AbortOn int
@@ -50,7 +52,7 @@ type HTTPRunnerResults struct {
 
 // Run tests http request fetching. Main call being run at the target QPS.
 // To be set as the Function in RunnerOptions.
-func (httpstate *HTTPRunnerResults) Run(t int) {
+func (httpstate *HTTPRunnerResults) Run(t int) (bool, string) {
 	log.Debugf("Calling in %d", t)
 	code, body, headerSize := httpstate.client.Fetch()
 	size := len(body)
@@ -62,6 +64,10 @@ func (httpstate *HTTPRunnerResults) Run(t int) {
 		httpstate.aborter.Abort()
 		log.Infof("Aborted run because of code %d - data %s", code, DebugSummary(body, 1024))
 	}
+	if code == http.StatusOK {
+		return true, "200"
+	}
+	return false, fmt.Sprint(code)
 }
 
 // HTTPRunnerOptions includes the base RunnerOptions plus http specific
@@ -76,23 +82,30 @@ type HTTPRunnerOptions struct {
 }
 
 // RunHTTPTest runs an http test and returns the aggregated stats.
+// nolint: funlen, gocognit, gocyclo
 func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 	o.RunType = "HTTP"
-	log.Infof("Starting http test for %s with %d threads at %.1f qps", o.URL, o.NumThreads, o.QPS)
+	warmupMode := "parallel"
+	if o.SequentialWarmup {
+		warmupMode = "sequential"
+	}
+	log.Infof("Starting http test for %s with %d threads at %.1f qps and %s warmup", o.URL, o.NumThreads, o.QPS, warmupMode)
 	r := periodic.NewPeriodicRunner(&o.RunnerOptions)
 	defer r.Options().Abort()
 	numThreads := r.Options().NumThreads
 	o.HTTPOptions.Init(o.URL)
 	out := r.Options().Out // Important as the default value is set from nil to stdout inside NewPeriodicRunner
 	total := HTTPRunnerResults{
+		HTTPOptions: o.HTTPOptions,
 		RetCodes:    make(map[int]int64),
 		sizes:       stats.NewHistogram(0, 100),
 		headerSizes: stats.NewHistogram(0, 5),
-		URL:         o.URL,
 		AbortOn:     o.AbortOn,
 		aborter:     r.Options().Stop,
 	}
 	httpstate := make([]HTTPRunnerResults, numThreads)
+	// First build all the clients sequentially. This ensures we do not have data races when
+	// constructing requests.
 	for i := 0; i < numThreads; i++ {
 		r.Options().Runners[i] = &httpstate[i]
 		// Temp mutate the option so each client gets a logging id
@@ -104,7 +117,7 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 		if err != nil {
 			return nil, err
 		}
-		if o.Exactly <= 0 {
+		if o.SequentialWarmup && o.Exactly <= 0 {
 			code, data, headerSize := httpstate[i].client.Fetch()
 			if !o.AllowInitialErrors && !codeIsOK(code) {
 				return nil, fmt.Errorf("error %d for %s: %q", code, o.URL, string(data))
@@ -120,6 +133,25 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 		httpstate[i].AbortOn = total.AbortOn
 		httpstate[i].aborter = total.aborter
 	}
+	if o.Exactly <= 0 && !o.SequentialWarmup {
+		warmup := errgroup{}
+		for i := 0; i < numThreads; i++ {
+			i := i
+			warmup.Go(func() error {
+				code, data, headerSize := httpstate[i].client.Fetch()
+				if !o.AllowInitialErrors && !codeIsOK(code) {
+					return fmt.Errorf("error %d for %s: %q", code, o.URL, string(data))
+				}
+				if i == 0 && log.LogVerbose() {
+					log.LogVf("first hit of url %s: status %03d, headers %d, total %d\n%s\n", o.URL, code, headerSize, len(data), data)
+				}
+				return nil
+			})
+		}
+		if err := warmup.Wait(); err != nil {
+			return nil, err
+		}
+	}
 	// TODO avoid copy pasta with grpcrunner
 	if o.Profiler != "" {
 		fc, err := os.Create(o.Profiler + ".cpu")
@@ -165,8 +197,10 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 	r.Options().ReleaseRunners()
 	sort.Ints(keys)
 	totalCount := float64(total.DurationHistogram.Count)
-	_, _ = fmt.Fprintf(out, "Sockets used: %d (for perfect keepalive, would be %d)\n", total.SocketCount, r.Options().NumThreads)
-	_, _ = fmt.Fprintf(out, "Jitter: %t\n", total.Jitter)
+	if !o.DisableFastClient {
+		_, _ = fmt.Fprintf(out, "Sockets used: %d (for perfect keepalive, would be %d)\n", total.SocketCount, r.Options().NumThreads)
+	}
+	_, _ = fmt.Fprintf(out, "Uniform: %t, Jitter: %t\n", total.Uniform, total.Jitter)
 	for _, k := range keys {
 		_, _ = fmt.Fprintf(out, "Code %3d : %d (%.1f %%)\n", k, total.RetCodes[k], 100.*float64(total.RetCodes[k])/totalCount)
 	}
@@ -181,3 +215,37 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 	}
 	return &total, nil
 }
+
+// A errgroup is a collection of goroutines working on subtasks that are part of
+// the same overall task.
+type errgroup struct {
+	wg sync.WaitGroup
+
+	errOnce sync.Once
+	err     error
+}
+
+// Wait blocks until all function calls from the Go method have returned, then
+// returns the first non-nil error (if any) from them.
+func (g *errgroup) Wait() error {
+	g.wg.Wait()
+	return g.err
+}
+
+// Go calls the given function in a new goroutine.
+//
+// The first call to return a non-nil error cancels the group; its error will be
+// returned by Wait.
+func (g *errgroup) Go(f func() error) {
+	g.wg.Add(1)
+
+	go func() {
+		defer g.wg.Done()
+
+		if err := f(); err != nil {
+			g.errOnce.Do(func() {
+				g.err = err
+			})
+		}
+	}()
+}
