@@ -167,13 +167,22 @@ var (
 
 	maxStreamsFlag = flag.Uint("grpc-max-streams", 0,
 		"MaxConcurrentStreams for the grpc server. Default (0) is to leave the option unset.")
-	jitterFlag = flag.Bool("jitter", false, "set to true to de-synchronize parallel clients' requests")
+	jitterFlag    = flag.Bool("jitter", false, "set to true to de-synchronize parallel clients' by 10%")
+	uniformFlag   = flag.Bool("uniform", false, "set to true to de-synchronize parallel clients' requests uniformly")
+	nocatchupFlag = flag.Bool("nocatchup", false,
+		"set to exact fixed qps and prevent fortio from trying to catchup when the target fails to keep up temporarily")
 	// nc mode flag(s).
 	ncDontStopOnCloseFlag = flag.Bool("nc-dont-stop-on-eof", false, "in netcat (nc) mode, don't abort as soon as remote side closes")
 	// Mirror origin global setting (should be per destination eventually).
 	mirrorOriginFlag = flag.Bool("multi-mirror-origin", true, "Mirror the request url to the target for multi proxies (-M)")
 	multiSerialFlag  = flag.Bool("multi-serial-mode", false, "Multi server (-M) requests one at a time instead of parallel mode")
 	udpTimeoutFlag   = flag.Duration("udp-timeout", udprunner.UDPTimeOutDefaultValue, "Udp timeout")
+
+	accessLogFileFlag = flag.String("access-log-file", "",
+		"file `path` to log all requests to. Maybe have performance impacts")
+	accessLogFileFormat = flag.String("access-log-format", "json",
+		"`format` for access log. Supported values: [json, influx]")
+	calcQPS = flag.Bool("calc-qps", false, "Calculate the qps based on number of requests (-n) and duration (-t)")
 )
 
 // nolint: funlen // well yes it's fairly big and lotsa ifs.
@@ -188,6 +197,10 @@ func main() {
 	command := os.Args[1]
 	os.Args = append([]string{os.Args[0]}, os.Args[2:]...)
 	flag.Parse()
+	if *bincommon.HelpFlag {
+		usage(os.Stdout)
+		os.Exit(0)
+	}
 	if *bincommon.QuietFlag {
 		log.SetLogLevelQuiet(log.Error)
 	}
@@ -335,7 +348,7 @@ func fortioNC() {
 	}
 }
 
-// nolint: funlen // maybe refactor/shorten later.
+// nolint: funlen, gocognit // maybe refactor/shorten later.
 func fortioLoad(justCurl bool, percList []float64) {
 	if len(flag.Args()) != 1 {
 		usageErr("Error: fortio load/curl needs a url or destination")
@@ -349,6 +362,13 @@ func fortioLoad(justCurl bool, percList []float64) {
 	prevGoMaxProcs := runtime.GOMAXPROCS(*goMaxProcsFlag)
 	out := os.Stderr
 	qps := *qpsFlag // TODO possibly use translated <=0 to "max" from results/options normalization in periodic/
+	if *calcQPS {
+		if *exactlyFlag == 0 || *durationFlag <= 0 {
+			usageErr("Error: can't use `-calc-qps` without also specifying `-n` and `-t`")
+		}
+		qps = float64(*exactlyFlag) / (*durationFlag).Seconds()
+		log.LogVf("Calculated QPS to do %d request in %v: %f", *exactlyFlag, *durationFlag, qps)
+	}
 	_, _ = fmt.Fprintf(out, "Fortio %s running at %g queries per second, %d->%d procs",
 		version.Short(), qps, prevGoMaxProcs, runtime.GOMAXPROCS(0))
 	if *exactlyFlag > 0 {
@@ -388,25 +408,29 @@ func fortioLoad(justCurl bool, percList []float64) {
 		Labels:      labels,
 		Exactly:     *exactlyFlag,
 		Jitter:      *jitterFlag,
+		Uniform:     *uniformFlag,
 		RunID:       *bincommon.RunIDFlag,
 		Offset:      *offsetFlag,
+		NoCatchUp:   *nocatchupFlag,
+	}
+	err := ro.AddAccessLogger(*accessLogFileFlag, *accessLogFileFormat)
+	if err != nil {
+		// Error already logged.
+		os.Exit(1)
 	}
 	var res periodic.HasRunnerResult
-	var err error
 	if *grpcFlag {
 		o := fgrpc.GRPCRunnerOptions{
 			RunnerOptions:      ro,
 			Destination:        url,
-			CACert:             *bincommon.CACertFlag,
-			Insecure:           bincommon.TLSInsecure(),
 			Service:            *healthSvcFlag,
 			Streams:            *streamsFlag,
 			AllowInitialErrors: *allowInitialErrorsFlag,
 			Payload:            httpOpts.PayloadString(),
 			Delay:              *pingDelayFlag,
 			UsePing:            *doPingLoadFlag,
-			UnixDomainSocket:   httpOpts.UnixDomainSocket,
 		}
+		o.TLSOptions = httpOpts.TLSOptions
 		res, err = fgrpc.RunGRPCTest(&o)
 	} else if strings.HasPrefix(url, tcprunner.TCPURLPrefix) {
 		o := tcprunner.RunnerOptions{
@@ -491,13 +515,12 @@ func grpcClient() {
 	if count <= 0 {
 		count = 1
 	}
-	cert := *bincommon.CACertFlag
+	httpOpts := bincommon.SharedHTTPOptions()
 	var err error
 	if *doHealthFlag {
-		_, err = fgrpc.GrpcHealthCheck(host, cert, *healthSvcFlag, count, bincommon.TLSInsecure())
+		_, err = fgrpc.GrpcHealthCheck(host, *healthSvcFlag, count, &httpOpts.TLSOptions)
 	} else {
-		httpOpts := bincommon.SharedHTTPOptions()
-		_, err = fgrpc.PingClientCall(host, cert, count, httpOpts.PayloadString(), *pingDelayFlag, httpOpts.Insecure)
+		_, err = fgrpc.PingClientCall(host, count, httpOpts.PayloadString(), *pingDelayFlag, &httpOpts.TLSOptions)
 	}
 	if err != nil {
 		// already logged
