@@ -29,6 +29,7 @@ import (
 	"os"
 	"os/signal"
 	"runtime"
+	"strings"
 	"sync"
 	"time"
 
@@ -51,7 +52,10 @@ var DefaultRunnerOptions = RunnerOptions{
 
 // Runnable are the function to run periodically.
 type Runnable interface {
-	Run(tid int)
+	// Run returns a boolean, true for normal/success, false otherwise.
+	// with details being an optional string that can be put in the access logs.
+	// Statistics are split into two sets.
+	Run(tid int) (status bool, details string)
 }
 
 // MakeRunners creates an array of NumThreads identical Runnable instances
@@ -132,10 +136,19 @@ type RunnerOptions struct {
 	// Enabling jitter (+/-10%) allows these requests to be de-synchronized
 	// When enabled, it is only effective in the '-qps' mode.
 	Jitter bool
+	// When multiple clients are used to generate requests, they tend to send
+	// requests very close to one another, causing a thundering herd problem
+	// Enabling uniform causes the requests between connections to be uniformly staggered.
+	// When enabled, it is only effective in the '-qps' mode.
+	Uniform bool
 	// Optional run id; used by the server to identify runs.
 	RunID int64
-	// Optional Offect Duration; to offset the histogram function duration
+	// Optional Offset Duration; to offset the histogram function duration
 	Offset time.Duration
+	// Optional AccessLogger to log every request made. See AddAccessLogger.
+	AccessLogger AccessLogger
+	// No catch-up: if true we will do exactly the requested QPS and not try to catch up if the target is temporarily slow.
+	NoCatchUp bool
 }
 
 // RunnerResults encapsulates the actual QPS observed and duration histogram.
@@ -149,10 +162,16 @@ type RunnerResults struct {
 	ActualDuration    time.Duration
 	NumThreads        int
 	Version           string
+	// DurationHistogram all the Run. If you want to exclude the error cases; subtract ErrorsDurationHistogram to each bucket.
 	DurationHistogram *stats.HistogramData
-	Exactly           int64 // Echo back the requested count
-	Jitter            bool
-	RunID             int64 // Echo back the optional run id.
+	// ErrorsDurationHistogram is the durations of the error (Run returning false) cases.
+	ErrorsDurationHistogram *stats.HistogramData
+	Exactly                 int64 // Echo back the requested count
+	Jitter                  bool
+	Uniform                 bool
+	NoCatchUp               bool
+	RunID                   int64 // Echo back the optional run id
+	AccessLoggerInfo        string
 }
 
 // HasRunnerResult is the interface implictly implemented by HTTPRunnerResults
@@ -302,7 +321,7 @@ func (r *periodicRunner) Options() *RunnerOptions {
 	return &r.RunnerOptions // sort of returning this here
 }
 
-func (r *periodicRunner) runQPSSetup() (requestedDuration string, requestedQPS string, numCalls int64, leftOver int64) {
+func (r *periodicRunner) runQPSSetup(extra string) (requestedDuration string, requestedQPS string, numCalls int64, leftOver int64) {
 	// r.Duration will be 0 if endless flag has been provided. Otherwise it will have the provided duration time.
 	hasDuration := (r.Duration > 0)
 	// r.Exactly is > 0 if we use Exactly iterations instead of the duration.
@@ -311,8 +330,8 @@ func (r *periodicRunner) runQPSSetup() (requestedDuration string, requestedQPS s
 	requestedQPS = fmt.Sprintf("%.9g", r.QPS)
 	if !hasDuration && !useExactly {
 		// Always print that as we need ^C to interrupt, in that case the user need to notice
-		_, _ = fmt.Fprintf(r.Out, "Starting at %g qps with %d thread(s) [gomax %d] until interrupted\n",
-			r.QPS, r.NumThreads, runtime.GOMAXPROCS(0))
+		_, _ = fmt.Fprintf(r.Out, "Starting at %g qps with %d thread(s) [gomax %d] until interrupted%s\n",
+			r.QPS, r.NumThreads, runtime.GOMAXPROCS(0), extra)
 		return
 	}
 	// else:
@@ -337,27 +356,27 @@ func (r *periodicRunner) runQPSSetup() (requestedDuration string, requestedQPS s
 	if useExactly {
 		leftOver = r.Exactly - totalCalls
 		if log.Log(log.Warning) {
-			_, _ = fmt.Fprintf(r.Out, "Starting at %g qps with %d thread(s) [gomax %d] : exactly %d, %d calls each (total %d + %d)\n",
-				r.QPS, r.NumThreads, runtime.GOMAXPROCS(0), r.Exactly, numCalls, totalCalls, leftOver)
+			_, _ = fmt.Fprintf(r.Out, "Starting at %g qps with %d thread(s) [gomax %d] : exactly %d, %d calls each (total %d + %d)%s\n",
+				r.QPS, r.NumThreads, runtime.GOMAXPROCS(0), r.Exactly, numCalls, totalCalls, leftOver, extra)
 		}
 	} else {
 		if log.Log(log.Warning) {
-			_, _ = fmt.Fprintf(r.Out, "Starting at %g qps with %d thread(s) [gomax %d] for %v : %d calls each (total %d)\n",
-				r.QPS, r.NumThreads, runtime.GOMAXPROCS(0), r.Duration, numCalls, totalCalls)
+			_, _ = fmt.Fprintf(r.Out, "Starting at %g qps with %d thread(s) [gomax %d] for %v : %d calls each (total %d)%s\n",
+				r.QPS, r.NumThreads, runtime.GOMAXPROCS(0), r.Duration, numCalls, totalCalls, extra)
 		}
 	}
 	return requestedDuration, requestedQPS, numCalls, leftOver
 }
 
-func (r *periodicRunner) runNoQPSSetup() (requestedDuration string, numCalls int64, leftOver int64) {
+func (r *periodicRunner) runMaxQPSSetup(extra string) (requestedDuration string, numCalls int64, leftOver int64) {
 	// r.Duration will be 0 if endless flag has been provided. Otherwise it will have the provided duration time.
 	hasDuration := (r.Duration > 0)
 	// r.Exactly is > 0 if we use Exactly iterations instead of the duration.
 	useExactly := (r.Exactly > 0)
 	if !useExactly && !hasDuration {
 		// Always log something when waiting for ^C
-		_, _ = fmt.Fprintf(r.Out, "Starting at max qps with %d thread(s) [gomax %d] until interrupted\n",
-			r.NumThreads, runtime.GOMAXPROCS(0))
+		_, _ = fmt.Fprintf(r.Out, "Starting at max qps with %d thread(s) [gomax %d] until interrupted%s\n",
+			r.NumThreads, runtime.GOMAXPROCS(0), extra)
 		return
 	}
 	// else:
@@ -370,12 +389,12 @@ func (r *periodicRunner) runNoQPSSetup() (requestedDuration string, numCalls int
 		numCalls = r.Exactly / int64(r.NumThreads)
 		leftOver = r.Exactly % int64(r.NumThreads)
 		if log.Log(log.Warning) {
-			_, _ = fmt.Fprintf(r.Out, "for %s (%d per thread + %d)\n", requestedDuration, numCalls, leftOver)
+			_, _ = fmt.Fprintf(r.Out, "for %s (%d per thread + %d)%s\n", requestedDuration, numCalls, leftOver, extra)
 		}
 	} else {
 		requestedDuration = fmt.Sprint(r.Duration)
 		if log.Log(log.Warning) {
-			_, _ = fmt.Fprintf(r.Out, "for %s\n", requestedDuration)
+			_, _ = fmt.Fprintf(r.Out, "for %s%s\n", requestedDuration, extra)
 		}
 	}
 	return
@@ -384,7 +403,7 @@ func (r *periodicRunner) runNoQPSSetup() (requestedDuration string, numCalls int
 // Run starts the runner.
 func (r *periodicRunner) Run() RunnerResults {
 	r.Stop.Lock()
-	runnerChan := r.Stop.StopChan // need a copy to not race with assignement to nil
+	runnerChan := r.Stop.StopChan // need a copy to not race with assignment to nil
 	r.Stop.Unlock()
 	useQPS := (r.QPS > 0)
 	// r.Exactly is > 0 if we use Exactly iterations instead of the duration.
@@ -392,11 +411,16 @@ func (r *periodicRunner) Run() RunnerResults {
 	var numCalls int64
 	var leftOver int64 // left over from r.Exactly / numThreads
 	var requestedDuration string
+	// AccessLogger info check
+	extra := ""
+	if r.AccessLogger != nil {
+		extra = fmt.Sprintf(" with access logger %s", r.AccessLogger.Info())
+	}
 	requestedQPS := "max"
 	if useQPS {
-		requestedDuration, requestedQPS, numCalls, leftOver = r.runQPSSetup()
+		requestedDuration, requestedQPS, numCalls, leftOver = r.runQPSSetup(extra)
 	} else {
-		requestedDuration, numCalls, leftOver = r.runNoQPSSetup()
+		requestedDuration, numCalls, leftOver = r.runMaxQPSSetup(extra)
 	}
 	runnersLen := len(r.Runners)
 	if runnersLen == 0 {
@@ -409,19 +433,21 @@ func (r *periodicRunner) Run() RunnerResults {
 	start := time.Now()
 	// Histogram  and stats for Function duration - millisecond precision
 	functionDuration := stats.NewHistogram(r.Offset.Seconds(), r.Resolution)
+	errorsDuration := stats.NewHistogram(r.Offset.Seconds(), r.Resolution)
 	// Histogram and stats for Sleep time (negative offset to capture <0 sleep in their own bucket):
 	sleepTime := stats.NewHistogram(-0.001, 0.001)
 	if r.NumThreads <= 1 {
 		log.LogVf("Running single threaded")
-		runOne(0, runnerChan, functionDuration, sleepTime, numCalls+leftOver, start, r)
+		runOne(0, runnerChan, functionDuration, errorsDuration, sleepTime, numCalls+leftOver, start, r)
 	} else {
 		var wg sync.WaitGroup
-		var fDs []*stats.Histogram
-		var sDs []*stats.Histogram
+		var fDs, eDs, sDs []*stats.Histogram
 		for t := 0; t < r.NumThreads; t++ {
 			durP := functionDuration.Clone()
+			errP := errorsDuration.Clone()
 			sleepP := sleepTime.Clone()
 			fDs = append(fDs, durP)
+			eDs = append(eDs, errP)
 			sDs = append(sDs, sleepP)
 			wg.Add(1)
 			thisNumCalls := numCalls
@@ -429,14 +455,15 @@ func (r *periodicRunner) Run() RunnerResults {
 				// The first thread gets to do the additional work
 				thisNumCalls += leftOver
 			}
-			go func(t int, durP *stats.Histogram, sleepP *stats.Histogram) {
-				runOne(t, runnerChan, durP, sleepP, thisNumCalls, start, r)
+			go func(t int, durP, errP, sleepP *stats.Histogram) {
+				runOne(t, runnerChan, durP, errP, sleepP, thisNumCalls, start, r)
 				wg.Done()
-			}(t, durP, sleepP)
+			}(t, durP, errP, sleepP)
 		}
 		wg.Wait()
 		for t := 0; t < r.NumThreads; t++ {
 			functionDuration.Transfer(fDs[t])
+			errorsDuration.Transfer(eDs[t])
 			sleepTime.Transfer(sDs[t])
 		}
 	}
@@ -465,18 +492,25 @@ func (r *periodicRunner) Run() RunnerResults {
 	if useExactly && actualCount != r.Exactly {
 		requestedDuration += fmt.Sprintf(", interrupted after %d", actualCount)
 	}
+	var loggerInfo string
+	if r.AccessLogger != nil {
+		loggerInfo = r.AccessLogger.Info()
+	}
 	result := RunnerResults{
 		r.RunType, r.Labels, start, requestedQPS, requestedDuration,
 		actualQPS, elapsed, r.NumThreads, version.Short(), functionDuration.Export().CalcPercentiles(r.Percentiles),
-		r.Exactly, r.Jitter, r.RunID,
+		errorsDuration.Export().CalcPercentiles(r.Percentiles),
+		r.Exactly, r.Jitter, r.Uniform, r.NoCatchUp, r.RunID, loggerInfo,
 	}
 	if log.Log(log.Warning) {
 		result.DurationHistogram.Print(r.Out, "Aggregated Function Time")
+		result.ErrorsDurationHistogram.Print(r.Out, "Error cases")
 	} else {
 		functionDuration.Counter.Print(r.Out, "Aggregated Function Time")
 		for _, p := range result.DurationHistogram.Percentiles {
 			_, _ = fmt.Fprintf(r.Out, "# target %g%% %.6g\n", p.Percentile, p.Value)
 		}
+		errorsDuration.Counter.Print(r.Out, "Error cases")
 	}
 	select {
 	case <-runnerChan: // nothing
@@ -488,18 +522,127 @@ func (r *periodicRunner) Run() RunnerResults {
 	return result
 }
 
+// AccessLoggerType is the possible formats of the access logger (ACCESS_JSON or ACCESS_INFLUX).
+type AccessLoggerType int
+
+const (
+	// AccessJSON for json format of access log: {"latency":%f,"timestamp":%d,"thread":%d}.
+	AccessJSON AccessLoggerType = iota
+	// AccessInflux of influx format of access log.
+	// https://docs.influxdata.com/influxdb/v2.2/reference/syntax/line-protocol/
+	AccessInflux
+)
+
+func (t AccessLoggerType) String() string {
+	if t == AccessJSON {
+		return "json"
+	}
+	return "influx"
+}
+
+type fileAccessLogger struct {
+	mu     sync.Mutex
+	file   *os.File
+	format AccessLoggerType
+	info   string
+}
+
+// AccessLogger defines an interface to report a single request.
+type AccessLogger interface {
+	// Report logs a single request to a file.
+	Report(thread int, time int64, latency float64, status bool, details string)
+	Info() string
+}
+
+// AddAccessLogger adds an AccessLogger that writes to the provided file in the provided format.
+func (r *RunnerOptions) AddAccessLogger(filePath, format string) error {
+	if filePath == "" {
+		return nil
+	}
+	al, err := NewFileAccessLogger(filePath, format)
+	if err != nil {
+		// Error already logged
+		return err
+	}
+	r.AccessLogger = al
+	return nil
+}
+
+// NewFileAccessLogger creates an AccessLogger that writes to the provided file in the provided format.
+func NewFileAccessLogger(filePath, format string) (AccessLogger, error) {
+	var t AccessLoggerType
+	fl := strings.ToLower(format)
+	if fl == "json" {
+		t = AccessJSON
+	} else if fl == "influx" {
+		t = AccessInflux
+	} else {
+		err := fmt.Errorf("invalid format %q, should be \"json\" or \"influx\"", format)
+		log.Errf("%v", err)
+		return nil, err
+	}
+	return NewFileAccessLoggerByType(filePath, t)
+}
+
+// NewFileAccessLoggerByType creates an AccessLogger that writes to the file in the AccessLoggerType enum format.
+func NewFileAccessLoggerByType(filePath string, accessType AccessLoggerType) (AccessLogger, error) {
+	f, err := os.OpenFile(filePath, os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0o644)
+	if err != nil {
+		log.Errf("Unable to open access log %s: %v", filePath, err)
+		return nil, err
+	}
+	infoStr := fmt.Sprintf("mode %s to %s", accessType.String(), filePath)
+	return &fileAccessLogger{file: f, format: accessType, info: infoStr}, nil
+}
+
+// Report logs a single request to a file.
+func (a *fileAccessLogger) Report(thread int, time int64, latency float64, status bool, details string) {
+	a.mu.Lock()
+	switch a.format {
+	case AccessInflux:
+		// https://docs.influxdata.com/influxdb/v2.2/reference/syntax/line-protocol/
+		fmt.Fprintf(a.file, "latency,thread=%d,ok=%t value=%f,details=%q %d\n", thread, status, latency, details, time)
+	case AccessJSON:
+		fmt.Fprintf(a.file, "{\"latency\":%f,\"timestamp\":%d,\"thread\":%d,\"ok\":%t,\"details\":%q}\n",
+			latency, time, thread, status, details)
+	}
+	a.mu.Unlock()
+}
+
+// Info is used to print information about the logger.
+func (a *fileAccessLogger) Info() string {
+	return a.info
+}
+
 // runOne runs in 1 go routine (or main one when -c 1 == single threaded mode).
-// nolint: gocognit // we should try to simplify it though.
-func runOne(id int, runnerChan chan struct{},
-	funcTimes *stats.Histogram, sleepTimes *stats.Histogram, numCalls int64, start time.Time, r *periodicRunner) {
+// nolint: gocognit, gocyclo // we should try to simplify it though.
+func runOne(id int, runnerChan chan struct{}, funcTimes, errTimes, sleepTimes *stats.Histogram,
+	numCalls int64, start time.Time, r *periodicRunner,
+) {
 	var i int64
 	endTime := start.Add(r.Duration)
 	tIDStr := fmt.Sprintf("T%03d", id)
 	perThreadQPS := r.QPS / float64(r.NumThreads)
 	useQPS := (perThreadQPS > 0)
+
 	hasDuration := (r.Duration > 0)
 	useExactly := (r.Exactly > 0)
 	f := r.Runners[id]
+	if useQPS && r.Uniform {
+		delayBetweenRequest := 1. / perThreadQPS
+		// When using uniform mode, we should wait a bit relative to our QPS and thread ID.
+		// For example, with 10 threads and 1 QPS, thread 8 should delay 0.7s.
+		delaySeconds := delayBetweenRequest - (delayBetweenRequest / float64(r.NumThreads) * float64(r.NumThreads-id))
+		delayDuration := time.Duration(delaySeconds * float64(time.Second))
+		start = start.Add(delayDuration)
+		log.Debugf("%s sleep %v for uniform distribution", tIDStr, delayDuration)
+		select {
+		case <-runnerChan:
+			return
+		case <-time.After(delayDuration):
+			// continue normal execution
+		}
+	}
 
 MainLoop:
 	for {
@@ -516,38 +659,54 @@ MainLoop:
 				break
 			}
 		}
-		f.Run(id)
-		funcTimes.Record(time.Since(fStart).Seconds())
-		i++
+		status, details := f.Run(id)
+		latency := time.Since(fStart).Seconds()
+		if r.AccessLogger != nil {
+			r.AccessLogger.Report(id, fStart.UnixNano(), latency, status, details)
+		}
+		funcTimes.Record(latency)
+		if !status {
+			errTimes.Record(latency)
+		}
 		// if using QPS / pre calc expected call # mode:
 		if useQPS { // nolint: nestif
-			if (useExactly || hasDuration) && i >= numCalls {
-				break // expected exit for that mode
-			}
-			elapsed := time.Since(start)
-			var targetElapsedInSec float64
-			if hasDuration {
-				// This next line is tricky - such as for 2s duration and 1qps there is 1
-				// sleep of 2s between the 2 calls and for 3qps in 1sec 2 sleep of 1/2s etc
-				targetElapsedInSec = (float64(i) + float64(i)/float64(numCalls-1)) / perThreadQPS
-			} else {
-				// Calculate the target elapsed when in endless execution
-				targetElapsedInSec = float64(i) / perThreadQPS
-			}
-			targetElapsedDuration := time.Duration(int64(targetElapsedInSec * 1e9))
-			sleepDuration := targetElapsedDuration - elapsed
-			if r.Jitter {
-				sleepDuration += getJitter(sleepDuration)
-			}
-			log.Debugf("%s target next dur %v - sleep %v", tIDStr, targetElapsedDuration, sleepDuration)
-			sleepTimes.Record(sleepDuration.Seconds())
-			select {
-			case <-runnerChan:
-				break MainLoop
-			case <-time.After(sleepDuration):
-				// continue normal execution
+			for {
+				i++
+				if (useExactly || hasDuration) && i >= numCalls {
+					break MainLoop // expected exit for that mode
+				}
+				var targetElapsedInSec float64
+				if hasDuration {
+					// This next line is tricky - such as for 2s duration and 1qps there is 1
+					// sleep of 2s between the 2 calls and for 3qps in 1sec 2 sleep of 1/2s etc
+					targetElapsedInSec = (float64(i) + float64(i)/float64(numCalls-1)) / perThreadQPS
+				} else {
+					// Calculate the target elapsed when in endless execution
+					targetElapsedInSec = float64(i) / perThreadQPS
+				}
+				targetElapsedDuration := time.Duration(int64(targetElapsedInSec * 1e9))
+				elapsed := time.Since(start)
+				sleepDuration := targetElapsedDuration - elapsed
+				if r.NoCatchUp && sleepDuration < 0 {
+					// Skip that request as we took too long
+					log.LogVf("%s request took too long %.04f s, would sleep %v, skipping iter %d", tIDStr, latency, sleepDuration, i)
+					continue
+				}
+				if r.Jitter {
+					sleepDuration += getJitter(sleepDuration)
+				}
+				log.Debugf("%s target next dur %v - sleep %v", tIDStr, targetElapsedDuration, sleepDuration)
+				sleepTimes.Record(sleepDuration.Seconds())
+				select {
+				case <-runnerChan:
+					break MainLoop
+				case <-time.After(sleepDuration):
+					// continue normal execution
+				}
+				break // NoCatchUp false or sleepDuration > 0
 			}
 		} else { // Not using QPS
+			i++
 			if useExactly && i >= numCalls {
 				break
 			}
