@@ -18,9 +18,7 @@ package ui // import "fortio.org/fortio/ui"
 import (
 	"bytes"
 	"context"
-
-	// nolint: gosec // md5 is mandated, not our choice
-	"crypto/md5"
+	"crypto/md5" // nolint: gosec // md5 is mandated, not our choice
 	"embed"
 	"encoding/base64"
 	"encoding/json"
@@ -67,7 +65,8 @@ var (
 	uiPath      string // absolute (base)
 	logoPath    string // relative
 	chartJSPath string // relative
-	debugPath   string // mostly relative
+	debugPath   string // absolute
+	echoPath    string // absolute
 	fetchPath   string // this one is absolute
 	// Used to construct default URL to self.
 	urlHostPort string
@@ -116,7 +115,7 @@ const (
 )
 
 // Handler is the main UI handler creating the web forms and processing them.
-// nolint: funlen, gocognit, gocyclo, nestif // should be refactored indeed (TODO)
+// nolint: funlen, gocognit, gocyclo, nestif, maintidx // should be refactored indeed (TODO)
 func Handler(w http.ResponseWriter, r *http.Request) {
 	fhttp.LogRequest(r, "UI")
 	mode := menu
@@ -148,10 +147,13 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 	qps, _ := strconv.ParseFloat(r.FormValue("qps"), 64)
 	durStr := r.FormValue("t")
 	jitter := (r.FormValue("jitter") == "on")
+	uniform := (r.FormValue("uniform") == "on")
+	nocatchup := (r.FormValue("nocatchup") == "on")
 	grpcSecure := (r.FormValue("grpc-secure") == "on")
 	grpcPing := (r.FormValue("ping") == "on")
 	grpcPingDelay, _ := time.ParseDuration(r.FormValue("grpc-ping-delay"))
 	stdClient := (r.FormValue("stdclient") == "on")
+	sequentialWarmup := (r.FormValue("sequential-warmup") == "on")
 	httpsInsecure := (r.FormValue("https-insecure") == "on")
 	resolve := r.FormValue("resolve")
 	timeoutStr := strings.TrimSpace(r.FormValue("timeout"))
@@ -192,6 +194,8 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 		Labels:      labels,
 		Exactly:     n,
 		Jitter:      jitter,
+		Uniform:     uniform,
+		NoCatchUp:   nocatchup,
 	}
 	if mode == run {
 		ro.Normalize()
@@ -209,6 +213,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 	defaultHeaders := httpopts.AllHeaders()
 	httpopts.ResetHeaders()
 	httpopts.DisableFastClient = stdClient
+	httpopts.SequentialWarmup = sequentialWarmup
 	httpopts.Insecure = httpsInsecure
 	httpopts.Resolve = resolve
 	if len(payload) > 0 {
@@ -237,6 +242,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 			Version                     string
 			LogoPath                    string
 			DebugPath                   string
+			EchoDebugPath               string
 			ChartJSPath                 string
 			StartTime                   string
 			TargetURL                   string
@@ -248,7 +254,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 			DoStop                      bool
 			DoLoad                      bool
 		}{
-			r, defaultHeaders, version.Short(), logoPath, debugPath, chartJSPath,
+			r, defaultHeaders, version.Short(), logoPath, debugPath, echoPath, chartJSPath,
 			startTime.Format(time.ANSIC), url, labels, runid,
 			fhttp.RoundDuration(time.Since(startTime)), durSeconds, urlHostPort, mode == stop, mode == run,
 		})
@@ -285,8 +291,8 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 				Destination:   url,
 				UsePing:       grpcPing,
 				Delay:         grpcPingDelay,
-				Insecure:      httpsInsecure,
 			}
+			o.TLSOptions = httpopts.TLSOptions
 			if grpcSecure {
 				o.Destination = fhttp.AddHTTPS(url)
 			}
@@ -893,7 +899,8 @@ func Serve(baseurl, port, debugpath, uipath, datadir string, percentileList []fl
 		log.Warnf("Adding missing trailing / to UI path '%s'", uiPath)
 		uiPath += "/"
 	}
-	debugPath = ".." + debugpath // TODO: calculate actual path if not same number of directories
+	debugPath = debugpath
+	echoPath = fhttp.EchoDebugPath(debugpath)
 	mux.HandleFunc(uiPath, Handler)
 	fetchPath = uiPath + fetchURI
 	// For backward compatibility with http:// only fetcher
